""" This file contains the logic for the IDGAF server. It can be used to make a stand-alone 
    IDGAF server, or it can be used as a module in a larger application. 
"""
import json
import random
import security
import re
import requests
from time import time
from datetime import datetime
import settings

""" The URL for the OpenAI API. """
URL = "https://api.openai.com/v1/completions"

""" The data to send to OpenAI. """
DATA = {"model": settings.OPEN_AI_COMPLETION_MODEL,
        "prompt": settings.PROMPTS[0],
        "temperature": settings.TEMPERATURE,
        "max_tokens": settings.OPEN_AI_MAX_TOKENS
        }

""" The headers to send to OpenAI. """
request_headers = {"Authorization": "Bearer " +
                   settings.APIKEY, "Content-Type": "application/json"}

def get_response_base_object(text:str) -> dict:
    """ This is used to create the response object for the server.
        Parameters:
            text: The text to return to the client.
        Returns:
            A dictionary containing the response object.
    """    
    resultObject = {}
    resultObject["message"] = {}
    resultObject["service"] = "AIDGAF Server"
    resultObject["message"]["data"] = {}
    resultObject["message"]["data"]["resultObject"] = text
    resultObject["timestamp"] = datetime.utcnow().timestamp()
    return resultObject


def parse_idgaf_request(command)->[int, dict]:
    """ This function handles the IDGAF command. It will return a response object.
        Parameters:
            command: The command object received from the client.
        Returns:
            A tuple containing the status code and the response object.
    """
    the_data = get_prompt(command)
    response = get_gpt_response(the_data)
    try:
        response_text = response.json()['choices'][0]['text'].strip()
    except (KeyError):
        response_text = gpt_response.text
    obj = get_response_base_object(response_text)
    return [response.status_code, obj]

def get_gpt_response(data) -> requests.Response:
    """ This function communicates with OpenAI and returns a response object.
        Parameters:
            data: The data to send to OpenAI.
        Returns:
            The response object from OpenAI.
    """
    gpt_response = requests.post(URL, json=data, headers=request_headers)
    return gpt_response

def get_prompt(command)->dict:
    """ Selects a prompt from the PROMPTS list and replaces the USERNAME placeholder with the username.
        Parameters:
            command: The command object received from the client.
        Returns:
            A dictionary containing the data to send to OpenAI.
    """
    my_prompt = random.choice(settings.PROMPTS)
    my_prompt = my_prompt.replace(
        "USERNAME", command['message']['data']['username'])

    print("Prompt selected: "+my_prompt)
    the_data = DATA
    the_data["prompt"] = my_prompt
    return the_data


if __name__ == "__main__":
    """ This function is for testing the IDGAF capabilities without a server. """
    value = '{"service":"papa","message":{"command":"aidgaf","data":{"username":"AdamOutler"},"timestamp":1675725191},"hash":"1bc73914478835d03f9ebdfb46328321d2bb656647e2876d6f162cc1860607fcfca8d825c48e390a6a254ee0835c8a4fe5f9a25795a3a0880ae5a23e9c132cf2"}'
    command = json.loads(value)
    [code, result] = parse_idgaf_request(command)
    print(result)
